@@ -6,7 +6,7 @@ use 5.016;
 use strict;
 use warnings;
 
-our $VERSION = '4.0';
+our $VERSION = '4.1.2';
 
 use SBO::Lib::Util qw/ :config :const :times :colors prompt error_code script_error get_sbo_from_loc check_multilib on_blacklist open_fh open_read uniq save_options wrapsay in in_regexp $userland_32 /;
 use SBO::Lib::Tree qw/ get_sbo_location /;
@@ -52,8 +52,9 @@ our @EXPORT_OK = qw{
   $tmpd
   $env_tmp
   @last_level_reverse
-  @concluded
+  %concluded
   @reverse_concluded
+  %warnings
 };
 
 our %EXPORT_TAGS = (
@@ -76,13 +77,12 @@ SBO::Lib::Build - Routines for building Slackware packages from SlackBuilds.org.
 
 =head1 VARIABLES
 
-=head2 @concluded
+=head2 %concluded
 
-This is a shared array that tracks scripts with verified
-completable build queues; it is used by C<get_build_queue()> to check for
-circular dependencies. Note that C<@concluded> is cleared
-by C<sbofind> and C<sboremove> between results, and by C<sboinstall> when
-running with C<mass-rebuild> or C<series-rebuild>.
+This is a shared hash that contains already-calculated build queues.
+It is used by C<get_build_queue()> to reduce wasted time and check for
+circular dependencies. Note that C<%concluded> is cleared
+by C<sbotool> when using the C<Refresh> button.
 
 =head2 $env_tmp
 
@@ -94,8 +94,7 @@ set.
 This is a shared array that tracks scripts with verified
 reverse dependency chains; it is used by C<get_full_reverse()> to check for
 circular reverse dependencies. Note that C<@reverse_concluded> is cleared
-by C<sbofind> and C<sboremove> between results, and by C<sboinstall> when
-running with C<mass-rebuild> or C<series-rebuild>.
+by C<sbofind>, C<sboremove> and C<sbotool> between results.
 
 =head2 $tempdir
 
@@ -118,6 +117,11 @@ This is a shared, non-exportable array that contains hashes with the source
 files needed by each script in the queue. Each hash drops out of the array
 when its corresponding script has been built.
 
+=head2 %warnings
+
+This is a shared hash that contains per-script warnings. Possible warnings
+currently include C<%README%> and C<nonexistent>.
+
 =cut
 
 # get $TMP from the env, if defined - we use two variables here because there
@@ -130,9 +134,10 @@ make_path($tmpd) unless -d $tmpd;
 our $tempdir;
 $tempdir = tempdir(CLEANUP => 1, DIR => $tmpd) if $< == 0;
 
-# this array tracks scripts that have verified completable build queues; it is
-# used to check for circular dependencies
-our @concluded;
+# this hash contains already-calculated build queues
+our %concluded;
+# this array is used to check for circular dependencies in reverse queue
+# calculations
 our @reverse_concluded;
 
 # this array keeps track of files needed by subsequent scripts
@@ -141,6 +146,9 @@ our @upcoming;
 # this array retains last-level reverse dependencies found by get_full_reverse
 our @last_level_reverse;
 
+# a universal warnings hash; warnings currently include README and nonexistence
+our %warnings;
+
 =head1 SUBROUTINES
 
 =cut
@@ -260,18 +268,18 @@ sub do_upgradepkg {
 
 =head2 get_build_queue
 
-  my @queue = @{ get_build_queue($sbo, my $warnings, my @checked) };
+  my @queue = @{ get_build_queue($sbo, my @checked) };
 
-C<get_build_queue()> gets the prerequisites for C<$sbo>, updating the
+C<get_build_queue()> gets the prerequisites for C<$sbo>, updating the shared
 C<$warnings> hash reference with any C<%README%> encountered. It returns the
 prerequisites and C<$sbo> in the correct build order.
 
-C<@checked> and C<our @concluded> are used to check for circular dependencies; the
+C<@checked> and C<our %concluded> are used to check for circular dependencies; the
 script exits with C<_ERR_CIRCULAR> if any are present.
 =cut
 
 sub get_build_queue {
-  script_error('get_build_queue requires two arguments.') unless @_ == 2;
+  script_error('get_build_queue requires an argument.') unless @_ == 1;
   return [ _build_queue(@_) ];
 }
 
@@ -281,7 +289,8 @@ sub get_build_queue {
 
 C<get_full_queue()> takes a list of installed SlackBuilds and an array
 of SlackBuilds to check. It returns a list of the checked SlackBuilds and
-their dependencies in reverse build order.
+their dependencies in reverse build order such that no SlackBuild appears
+after any of its dependencies.
 
 =cut
 
@@ -289,12 +298,11 @@ sub get_full_queue {
   my ($installed, @sbos) = @_;
 
   my $revdep_queue = [];
-  my %warnings;
   for my $sbo (@sbos) {
-    my $queue = get_build_queue([$sbo], \%warnings);
-    @$queue = reverse @$queue;
+    my $queue = get_build_queue([$sbo]);
     $revdep_queue = merge_queues($revdep_queue, $queue);
   }
+  @$revdep_queue = reverse @$revdep_queue;
 
   return map {; +{
       name => $_,
@@ -334,11 +342,10 @@ sub get_full_reverse {
   if (@sublist) {
     for my $revdep (@sublist) {
       next if in $revdep, @reverse_concluded;
-      my %warnings;
       # The first two conditions are prone to false positives if a script
       # and its dependency share a listed dependency; get_build_queue
       # makes certain in these cases.
-      if (in $revdep, @checked and in $revdep, get_build_queue([$sbo], \%warnings)) {
+      if (in $revdep, @checked and in $revdep, get_build_queue([$sbo])) {
         error_code("Circular dependency for $revdep detected. Exiting.", _ERR_CIRCULAR);
       }
       push @checked, $revdep;
@@ -358,14 +365,13 @@ sub get_full_reverse {
 
 =head2 get_full_reverse_queue
 
-  my (@reverse_queue, %warnings) = get_full_reverse_queue($from, $updates, $sbo ...)
-  my (@reverse_queue, %warnings) = get_full_reverse_queue($from, $self_include, $sbo ...)
+  my (@reverse_queue) = get_full_reverse_queue($from, $updates, $sbo ...)
+  my (@reverse_queue) = get_full_reverse_queue($from, $self_include, $sbo ...)
 
 C<get_full_reverse_queue()> takes the name of the script it is called from and any number
 of SlackBuilds. The second variable is the list of available upgrades (if called from
 B<sboupgrade(1)>), any true value if called from B<sboinstall(1) --reinstall> and a
-false value otherwise. The subroutine returns a queue for a reverse dependency rebuild
-and a warnings hash.
+false value otherwise. The subroutine returns a queue for a reverse dependency rebuild.
 
 =cut
 
@@ -382,12 +388,12 @@ sub get_full_reverse_queue {
   my $installed = +{ map {; $_->{name}, $_->{pkg} } @installed };
   my $fulldeps = get_reverse_reqs($installed);
 
-  my ($return_queue, %warnings);
+  my $return_queue;
   REVERSE: for my $sbo (@_) {
     # if called from sboinstall --reverse-rebuild without --reinstall,
     # need to skip dependees of other requested scripts
     if ($from eq "sboinstall" and not $self_include) {
-      my $check_queue = get_build_queue([$sbo], \%warnings);
+      my $check_queue = get_build_queue([$sbo]);
       @$check_queue = grep { !/^$sbo$/ } @$check_queue;
       for my $sbo2 (@_) { next REVERSE if in $sbo2, @$check_queue; }
     }
@@ -395,7 +401,7 @@ sub get_full_reverse_queue {
     my @full_reverse = get_full_reverse($sbo, $installed, $fulldeps);
     if (@full_reverse) {
       for my $revdep (@full_reverse) {
-        my $queue = get_build_queue([$revdep], \%warnings);
+        my $queue = get_build_queue([$revdep]);
         # for sboinstall, install items not in the reverse queue only
         # if missing; for sboupgrade, upgrade items not in the reverse
         # queue if upgradable.
@@ -419,7 +425,7 @@ sub get_full_reverse_queue {
     }
     $return_queue = merge_queues($return_queue, $interim_queue) if $interim_queue;
   }
-  return ($return_queue, %warnings) if $return_queue;
+  return $return_queue if $return_queue;
   return 0;
 }
 
@@ -867,40 +873,83 @@ sub process_sbos {
   my $queue = rationalize_queue($queue)
 
 C<rationalize_queue()> takes a build queue and rearranges it such that
-no script appears before any of its dependencies. Currently, this is only
-useful when an automatic reverse dependency rebuild has been triggered or
-in case of a mass or series rebuild. The rearranged queue is returned.
+no script appears before any of its dependencies. The order is predictable
+given the scripts included and favors keeping dependency chains together,
+especially beyond the first level; this makes output and build orders more
+intuitive.
+
+Currently, this is only useful when the queue has been constructed with
+reference to reverse dependencies, or, in the case of C<sbotest>, a full
+repository test or archive rebuild is needed.
+
+C<sbotest> has a separate subroutine that reduces the number of expected
+package installations and removals during test runs. Intuitiveness is not
+to be expected in C<sbotest> queues.
+
+The rearranged queue is returned.
 
 =cut
 
 sub rationalize_queue {
   script_error('rationalize_queue requires an argument.') unless @_ == 1;
-  my $queue = shift;
-  my @queue = sort @{ $queue };
-  my (%all_reqs, @have_requirements, @result_queue);
+  my $preliminary_queue = shift;
+  my @preliminary_queue = sort @{ $preliminary_queue };
+  my (@queue, %queued, %pushed, %any_reqs, %reqs, @result_queue);
 
-  FIRST: while (my $sbo = shift @queue) {
+  for my $sbo (@preliminary_queue) {
     my $requirements = get_requires($sbo);
     unless (defined $requirements) {
       push @result_queue, $sbo;
-      next FIRST;
+      next;
     }
     my @reqs = @{ $requirements };
-    unless ($reqs[0]) {
+    my $sbo_queue = get_build_queue([$sbo]);
+    pop @$sbo_queue;
+    unless (@$sbo_queue) {
       push @result_queue, $sbo;
-      next FIRST;
+      next;
     }
-    $all_reqs{$sbo} = \@reqs;
-    push @have_requirements, $sbo;
+    $reqs{$sbo}->{$_} = 1 for (@$sbo_queue);
+    $any_reqs{$sbo} = \@$sbo_queue;
+    push @queue, $sbo;
   }
-  SECOND: while (my $sbo = shift @have_requirements) {
-    for my $check (@{$all_reqs{$sbo}}) {
-      if (in $check, @have_requirements) {
-        push @have_requirements, $sbo;
-        next SECOND;
+  $queued{$_} = 1 for (@queue);
+  return [ @result_queue ] unless @queue;
+
+  for my $no_reqs (@result_queue) {
+    my @new_queue;
+    while (my $sbo = shift @queue) {
+      if (exists $reqs{$sbo}->{$no_reqs}) {
+        unshift @new_queue, $sbo;
+      } else {
+        push @new_queue, $sbo;
+      }
+    }
+    @queue = @new_queue;
+  }
+
+  FIRST: while (my $sbo = shift @queue) {
+    if (exists $any_reqs{$sbo}) {
+      for my $check (@{$any_reqs{$sbo}}) {
+        if (not exists $pushed{$check} and exists $queued{$check}) {
+          push @queue, $sbo;
+          next FIRST;
+        }
       }
     }
     push @result_queue, $sbo;
+    $pushed{$sbo} = 1;
+    if (@queue) {
+      my @new_queue;
+      while (my $cand = shift @queue) {
+        if (exists $reqs{$cand}->{$sbo}) {
+          unshift @new_queue, $cand;
+        } else {
+          push @new_queue, $cand;
+        }
+      }
+      @queue = @new_queue;
+    }
   }
 
   return [ @result_queue ];
@@ -1026,29 +1075,31 @@ sub _time_restart {
 }
 
 sub _build_queue {
-  my ($sbos, $warnings, @checked) = @_;
+  my ($sbos, @checked) = @_;
   my @queue;
   for my $cand (@$sbos) { push @queue, $cand unless on_blacklist($cand); }
   my @result;
 
   while (my $sbo = shift @queue) {
     next if $sbo eq "%README%";
-    next if in $sbo, @concluded;
+    if (exists $concluded{$sbo} or exists $warnings{$sbo}) {
+      push @result, @{$concluded{$sbo}}, $sbo unless exists $warnings{$sbo} and $warnings{$sbo} eq "nonexistent";
+      next;
+    }
     if (in $sbo, @checked) {
       error_code("Circular dependencies for $sbo detected. Exiting.", _ERR_CIRCULAR);
     }
     push @checked, $sbo;
     my $reqs = get_requires($sbo);
     if (defined $reqs) {
-      push @result, _build_queue($reqs, $warnings, @checked);
-      push @concluded, $sbo;
+      my @sbo_result = _build_queue($reqs, @checked);
+      push @result, @sbo_result;
+      $concluded{$sbo} = \@sbo_result;
       foreach my $req (@$reqs) {
-        $warnings->{$sbo}="%README%" if $req eq "%README%";
+        $warnings{$sbo}="%README%" if $req eq "%README%";
       }
-    }
-    else {
-      push @concluded, $sbo;
-      $warnings->{$sbo} = "nonexistent";
+    } else {
+      $warnings{$sbo} = "nonexistent";
     }
     push @result, $sbo;
   }
