@@ -6,7 +6,7 @@ use 5.016;
 use strict;
 use warnings;
 
-our $VERSION = '4.0';
+our $VERSION = '4.1.2';
 
 use SBO::Lib::Util qw/ :config :const build_cmp in script_error error_code open_read version_cmp /;
 use SBO::Lib::Tree qw/ get_sbo_location is_local /;
@@ -24,6 +24,8 @@ our @EXPORT_OK = qw{
   get_installed_packages
   get_local_outdated_versions
   get_removed_builds
+  $perl_pkg
+  $ruby_pkg
 };
 
 our %EXPORT_TAGS = (
@@ -31,6 +33,7 @@ our %EXPORT_TAGS = (
 );
 
 my ($all_pkgs, $std_pkgs, $sbo_pkgs, $dirty_pkgs);
+our ($perl_pkg, $ruby_pkg);
 
 =pod
 
@@ -46,6 +49,24 @@ SBO::Lib::Pkgs - Routines for interacting with the Slackware package database.
 
   my @installed_sbos = get_installed_packages('SBO');
 
+=cut
+
+=head1 VARIABLES
+
+=cut
+
+=head2 $perl_pkg
+
+The file name of the installed C<perl> package.
+
+=cut
+
+=head2 $ruby_pkg
+
+The file name of the installed C<ruby> package.
+
+=cut
+
 =head1 SUBROUTINES
 
 =cut
@@ -193,6 +214,8 @@ versions, full installed package names and creation times of the returned packag
 The default behavior is to retain the package lists for future calls; add a true value
 to the arguments to clear them instead. This is irrelevant when running C<sbotest>.
 
+The C<perl> and C<ruby> package file names are found at this time.
+
 =cut
 
 # pull an array of hashes, each hash containing the name and version of a
@@ -214,6 +237,8 @@ sub get_installed_packages {
     $pkg =~ s!^\Q$pkg_db/\E!!;
     my ($name, $version, $build) = $pkg =~ m#^([^/]+)-([^-]+)-[^-]+-([^-]+)$#
       or next;
+    $perl_pkg = $pkg if $name eq "perl";
+    $ruby_pkg = $pkg if $name eq "ruby";
     my $numbuild = $build;
     $numbuild =~ s/_SBo(|compat32)$//g ;
     my $created = strftime "%F, %H:%M:%S", localtime((stat "$pkg_db/$pkg")[10]);
