@@ -6,11 +6,12 @@ use 5.016;
 use strict;
 use warnings;
 
-our $VERSION = '4.0';
+our $VERSION = '4.1.2';
 
 use SBO::Lib::Util qw/ :config :const :colors error_code prompt usage_error get_slack_branch get_slack_version get_slack_version_url script_error open_fh open_read in in_regexp slurp wrapsay /;
 
 use Cwd;
+use File::Basename;
 use File::Copy;
 use File::Find;
 use File::Path qw/ make_path remove_tree /;
@@ -29,6 +30,7 @@ our @EXPORT_OK = qw{
   rsync_sbo_tree
   slackbuilds_or_fetch
   update_tree
+  verify_file
   verify_gpg
 
   $distfiles
@@ -66,12 +68,8 @@ downloaded sources are kept.
 =head2 $gpg_log
 
 C<$gpg_log> defaults to C</usr/sbo/gpg.log>, and it is where the output
-of the most recent C<gnupg> verification is kept.
-
-=head2 $gpg_obsolete_log
-
-C<$gpg_obsolete_log> defaults to C</usr/sbo/gpg-obsolete.log>, and it is where the output
-of the most recent C<gnupg> verification for the obsolete script list is kept.
+of the most recent C<gnupg> verification for the repository is kept. Other
+C<gnupg> logs are saved under C</usr/sbo/file_name_here.asc.log>.
 
 =head2 $repo_path
 
@@ -92,7 +90,6 @@ C<$repo_path> proceeds without prompting.
 our $distfiles = "$config{SBO_HOME}/distfiles";
 our $repo_path = "$config{SBO_HOME}/repo";
 our $gpg_log = "$config{SBO_HOME}/gpg.log";
-our $gpg_obsolete_log = "$config{SBO_HOME}/gpg-obsolete.log";
 our $slackbuilds_txt = "$repo_path/SLACKBUILDS.TXT";
 
 =head1 SUBROUTINES
@@ -206,8 +203,6 @@ sub check_repo {
         last if $extra_dir;
       }
       for my $cat (@categories) {
-        # The gis category was added in 14.1.
-        next if $cat eq "gis";
         $incomplete = 1 unless in $cat, @found_dirs;
         last if $incomplete;
       }
@@ -269,9 +264,7 @@ sub check_repo {
   my $bool = generate_slackbuilds_txt();
 
 C<generate_slackbuilds_txt()> generates a minimal C<SLACKBUILDS.TXT> for
-repositories that do not include this file. The file is also generated
-under Slackware 14.0 and 14.1, which contain an absent SlackBuild and can
-thereby cause unexpected failures.
+repositories that do not include this file.
 
 If the file cannot be opened for write, it returns a false value. Otherwise,
 it returns a true value.
@@ -319,35 +312,41 @@ sub generate_slackbuilds_txt {
 
 C<get_obsolete()> downloads a file from the C<sbotools> home page listing scripts that are
 known to have been added to Slackware -current under different names, or to be obsolete
-out-of-tree build dependencies. It is saved to C</etc/sbotools/obsolete>. C<gnupg> verification
-is performed if C<GPG_VERIFY> is C<TRUE>. There is no useful return value.
+out-of-tree build dependencies. It is saved to C</etc/sbotools/obsolete>. The C<perl> build
+history file, C</etc/sbotools/perl_vers>, is downloaded as well.
+
+C<gnupg> verification is performed if C<GPG_VERIFY> is C<TRUE>. There is no useful return
+value.
 
 =cut
 
 sub get_obsolete {
   my $cwd = getcwd();
-  my $link = "https://pghvlaans.github.io/sbotools/downloads/obsolete";
-  my $link_asc = "$link.asc";
-  my $obs_asc = "$obs_file.asc";
+  my $link = "https://pghvlaans.github.io/sbotools/downloads/";
   if (-d "$conf_dir") {
-    chdir $conf_dir;
-    move($obs_file, "$obs_file.bk") if -f $obs_file;
-    wrapsay "\nDownloading the obsolete script list from $link...\n";
-    unless (system('wget', '--tries=5', $link) == 0) {
-      move("$obs_file.bk", $obs_file) if -f "$obs_file.bk";
-    } else {
-      unlink "$obs_file.bk" if -f "$obs_file.bk";
-    }
-    if ($config{GPG_VERIFY} eq "TRUE") {
-      unlink $obs_asc if -f $obs_asc;
-      unless (system('wget', '--tries=5', $link_asc) == 0) {
+    for my $file ($obs_file, $perl_file) {
+      chdir $conf_dir;
+      move($file, "$file.bk") if -f $file;
+      my $dl_link = $link . basename($file);
+      wrapsay "\nDownloading $file from $dl_link...\n";
+      unless (system('wget', '--tries=5', $dl_link) == 0) {
+        move("$file.bk", $file) if -f "$file.bk";
+      } else {
+        unlink "$file.bk" if -f "$file.bk";
+      }
+      if ($config{GPG_VERIFY} eq "TRUE") {
+        my $file_asc = $file . ".asc";
+        my $link_asc = $dl_link . ".asc";
+        unlink $file_asc if -f $file_asc;
+        unless (system('wget', '--tries=5', $link_asc) == 0) {
+          chdir $cwd;
+          error_code("$link_asc could not be downloaded.", _ERR_DOWNLOAD);
+        }
         chdir $cwd;
-        error_code("$obs_asc could not be downloaded.", _ERR_DOWNLOAD);
+        verify_file($file_asc);
       }
       chdir $cwd;
-      verify_obsolete();
     }
-    chdir $cwd;
   }
   return;
 }
@@ -478,7 +477,7 @@ sub pull_sbo_tree {
     $res = git_sbo_tree($url);
     if ($res == 0) {
       if (prompt($color_lesser, "Sync from $url failed. Retry?", default => 'no')) {
-        generate_slackbuilds_txt() unless -s $slackbuilds_txt or versioncmp($sw_version, "14.1") != 1;
+        generate_slackbuilds_txt() unless -s $slackbuilds_txt;
         return pull_sbo_tree();
       }
     }
@@ -488,7 +487,7 @@ sub pull_sbo_tree {
 
   my $wanted = sub { chown 0, 0, $File::Find::name; };
   find($wanted, $repo_path) if -d $repo_path;
-  if ($res and (versioncmp($sw_version, "14.1") != 1 or not -s $slackbuilds_txt)) {
+  if ($res and not -s $slackbuilds_txt) {
     generate_slackbuilds_txt();
   }
   if ($config{OBSOLETE_CHECK} eq "TRUE") {
@@ -513,10 +512,7 @@ sub rsync_sbo_tree {
   script_error('rsync_sbo_tree requires an argument.') unless @_ == 1;
   my $url = shift;
   $url .= '/' unless $url =~ m!/$!; # make sure $url ends with /
-  my @info;
-  # only Slackware versions above 14.1 have an rsync that supports --info=progress2
-  if (versioncmp(get_slack_version(), '14.1') == 1) { @info = ('--info=progress2'); }
-  my @args = ('rsync', @info, '-a', '--delete', $url);
+  my @args = ('rsync', '--info=progress2', '-a', '--delete', $url);
   my $res = system(@args, $repo_path) == 0;
   if ($config{GPG_VERIFY} eq "TRUE") {
     return 0 unless $res;
@@ -584,21 +580,10 @@ sub update_tree {
 C<verify_git_commit()> attempts to verify the GPG signature of the most
 recent git commit, if any.
 
-Git commit verification is unavailable for Slackware 14.0 and Slackware 14.1.
-A user prompt for continuation appears if C<GPG_VERIFY> is C<TRUE>.
-
 =cut
 
 sub verify_git_commit {
   script_error('verify_git_commit requires an argument.') unless @_ == 1;
-  # verifying git commits is only supported for 14.2 onwards
-  if (versioncmp(get_slack_version(), '14.1') != 1) {
-    if (prompt($color_warn, "Git verification is unsupported for Slackware 14.0 and 14.1. Proceed anyway?", default => 'no')) {
-      return 1;
-    } else {
-      usage_error("Exiting. Consider using rsync or change GPG_VERIFY to FALSE.");
-    }
-  }
   my $branch = shift;
   my $res;
   {
@@ -683,25 +668,8 @@ sub verify_rsync {
   {
     open OLDERR, '>&', \*STDERR;
     open STDERR, '>', $gpg_log;
-    if (versioncmp(get_slack_version(), '14.0') == 1) {
-      $checksum_asc_ok = system(qw/ gpg --status-file /, $tempfile, qw/ --verify CHECKSUMS.md5.asc /) == 0;
-      wrapsay_color $color_notice, "CHECKSUMS.md5.asc verified. See $gpg_log." if $checksum_asc_ok;
-    } else {
-      # CHECKSUMS.md5.asc is unsigned in the 14.0 repository; check all .asc files
-      wrapsay_color $color_notice, "\nChecking .asc files...";
-      $checksum_asc_ok = system(qw/ gpg --status-file /, $tempfile, qw! --verify system/sbotools.tar.gz.asc !) == 0;
-      if ($checksum_asc_ok) {
-        my @ascs = split(' ', `find . -name "*.asc"`);
-        for my $asc (@ascs) {
-          my $ascres = system(qw/ gpg --verify /, $asc) == 0;
-          unless ($ascres) {
-            $checksum_asc_ok = 0;
-            last;
-          }
-        }
-      }
-      wrapsay_color $color_notice, ".asc files verified. See $gpg_log." if $checksum_asc_ok;
-    }
+    $checksum_asc_ok = system(qw/ gpg --status-file /, $tempfile, qw/ --verify CHECKSUMS.md5.asc /) == 0;
+    wrapsay_color $color_notice, "CHECKSUMS.md5.asc verified. See $gpg_log." if $checksum_asc_ok;
     close STDERR;
     open STDERR, '>&', \*OLDERR;
   }
@@ -744,29 +712,7 @@ sub verify_rsync {
       }
     }
     chdir $repo_path or return 0;
-    # --ignore-missing is only available in 14.2 onwards.
-    if(versioncmp(get_slack_version(), '14.1') == 1) {
-      $res = system("tail +13 CHECKSUMS.md5 | md5sum -c --ignore-missing --quiet -") == 0;
-    } else {
-      # Disregard missing files in 14.0 and 14.1 as well.
-      my $md5temp = "$repo_path/CHECKSUMS.temp.md5";
-      unlink $md5temp if -f $md5temp;
-      my ($temp_fh, $exit) = open_fh($md5temp, '>');
-      return 0 if $exit;
-      my @checksum_lines = split('\n', slurp("CHECKSUMS.md5"));
-      for my $checksum_line (@checksum_lines){
-        my $checksum_file = $checksum_line;
-        $checksum_file =~ s/^.*\s//s;
-        print { $temp_fh } "$checksum_line\n" if -f $checksum_file;
-      }
-      close $temp_fh;
-      if (-f $md5temp) {
-        $res = system(qw/ md5sum -c --quiet /, $md5temp) == 0;
-        unlink $md5temp;
-      } else {
-        $res = 0;
-      }
-    }
+    $res = system("tail +13 CHECKSUMS.md5 | md5sum -c --ignore-missing --quiet -") == 0;
     if ($res) {
       # All is well, so release the lock, if any.
       unlink($rsync_lock) if -f $rsync_lock;
@@ -818,27 +764,29 @@ sub verify_gpg {
   }
 }
 
-=head2 verify_obsolete
+=head2 verify_file
 
-  verify_obsolete();
+  verify_file($asc);
 
-C<verify_obsolete()> runs C<gnupg> verification on a newly-downloaded
-C</etc/sbotools/obsolete> file. There is no useful return value.
+C<verify_file()> runs C<gnupg> verification, taking an C<asc> file as the only
+argument. There is no useful return value.
 
 =cut
 
-sub verify_obsolete {
+sub verify_file {
+  script_error("verify_file requires an argument.") unless @_ == 1;
   my $gpg_ok;
-  my $obs_asc = "$obs_file.asc";
+  my $asc = shift;
+  my $log = $config{SBO_HOME} . "/" . basename($asc) . ".log";
   my ($fh, $tempfile) = tempfile(DIR => "$config{SBO_HOME}");
-  unlink $gpg_obsolete_log if -f $gpg_obsolete_log;
+  unlink $log if -f $log;
   open OLDERR, '>&', \*STDERR;
-  open STDERR, '>', $gpg_obsolete_log;
-  $gpg_ok = (system('gpg', '--status-file', $tempfile, '--verify', $obs_asc) == 0);
+  open STDERR, '>', $log;
+  $gpg_ok = (system('gpg', '--status-file', $tempfile, '--verify', $asc) == 0);
   close STDERR;
   open STDERR, '>&', \*OLDERR;
   if ($gpg_ok) {
-    wrapsay "$obs_asc verified. See $gpg_obsolete_log.";
+    wrapsay "$asc verified. See $log.";
     close $fh;
     unlink $tempfile if -f $tempfile;
     return;
@@ -858,7 +806,7 @@ sub verify_obsolete {
       $next = 1 if $word eq "ERRSIG";
     }
     my $newkey = retrieve_key($fingerprint);
-    return verify_obsolete() if $newkey;
+    return verify_file($asc) if $newkey;
   }
   # REVKEYSIG: warning and exit
   if (grep(/REVKEYSIG/, @raw)) {
@@ -884,7 +832,8 @@ C<hkp://keyserver.ubuntu.com:80> and add it to the keyring.
 
 C<gnupg> output is saved to C<$key_log>, and the output of
 C<gpg --no-batch --search-keys> is displayed with a prompt to ensure
-that the user can trust the key.
+that the user can trust the key. The script exits with C<_ERR_GPG> if
+the download is declined.
 
 =cut
 
@@ -915,7 +864,7 @@ sub retrieve_key {
   } else {
     close STDERR;
     open STDERR, '>&', \*OLDERR;
-    return 0;
+    error_code("GPG verification is not possible without this public key.", _ERR_GPG);
   }
 }
 
