@@ -2,22 +2,16 @@ package SBO::Lib::Util;
 
 # vim: ts=2:et
 
-# The lines below are used to inform the sbotest wrapper script
-# that sbotools is compatible with "sbotest config",
-# "sbotest hints" and "sbotest find". Do not alter or remove.
+# The line(s) below are used to inform the sbotest wrapper script
+# that sbotools is compatible.
 
-#% SBOTEST CONFIG HINTS %
-#% SBOTEST FIND %
-
-#% SBOTEST V1 COMPAT %
-#% SBOTEST V1.0.1 COMPAT %
-#% SBOTEST V1.1.1 COMPAT %
+#% SBOTEST V1.2 COMPAT %concluded %
 
 use 5.016;
 use strict;
 use warnings;
 
-our $VERSION = '4.0';
+our $VERSION = '4.1.2';
 
 use Exporter 'import';
 use File::Copy;
@@ -62,12 +56,16 @@ my @EXPORT_CONFIG = qw{
   $obs_file
   @obsolete
   %optional
+  $perl_file
   $pkg_db
+  $rem_pkg_db
   $sbotest_compatible
   $script_db
   $is_sbotest
   $is_sbotool
   $userland_32
+  @py_installed
+  @py_missing
 };
 
 my @EXPORT_TIME = qw{
@@ -257,6 +255,14 @@ interact with C<@obsolete> directly; in other situations, make a copy.
 
 =cut
 
+=head2 $perl_file
+
+This file contains build timestamps for the introduction of major perl versions
+into Slackware. It is used by C<Solibs.pm> during the C<perl> package test. It is
+located at C</etc/sbotools/perl_vers> by default.
+
+=cut
+
 =head2 $userland_32
 
 C<get_arch()> defines this variable when running in a 32-bit userland on a 64-bit
@@ -266,8 +272,9 @@ kernel.
 
 our $arch = get_arch();
 
-our $pkg_db = '/var/log/packages';
-our $script_db = '/var/log/scripts';
+our $pkg_db = '/var/lib/pkgtools/packages';
+our $rem_pkg_db = '/var/lib/pkgtools/removed_packages';
+our $script_db = '/var/lib/pkgtools/scripts';
 
 # global config variables
 my $req_dir = $ENV{SBOTOOLS_CONF_DIR};
@@ -281,6 +288,7 @@ my $filebase = defined $is_sbotest ? "sbotest" : "sbotools";
 our $conf_file = "$conf_dir/$filebase.conf";
 our $hint_file = "$conf_dir/$filebase.hints";
 our $obs_file = "$conf_dir/obsolete";
+our $perl_file = "$conf_dir/perl_vers";
 our $color_file = "$conf_dir/$filebase.colors";
 our %config = (
   CLASSIC => 'FALSE',
@@ -604,12 +612,8 @@ sub get_sbo_from_loc {
 }
 
 # %supported maps what's in /etc/slackware-version to an https URL, or to an
-# rsync URL if RSYNC_DEFAULT is true. Git commit verification is unavailable
-# prior to Slackware 14.2, so prior versions have rsync as well.
+# rsync URL if RSYNC_DEFAULT is true.
 my %supported = (
-  '14.0' => 'rsync://slackbuilds.org/slackbuilds/14.0/',
-  '14.1' => 'rsync://slackbuilds.org/slackbuilds/14.1/',
-  '14.2' => 'https://gitlab.com/SlackBuilds.org/slackbuilds.git',
   '15.0' => 'https://gitlab.com/SlackBuilds.org/slackbuilds.git',
   '15.0+' => 'https://github.com/Ponce/slackbuilds.git',
   '15.1' => 'https://gitlab.com/SlackBuilds.org/slackbuilds.git',
@@ -619,9 +623,6 @@ my %supported = (
 
 if ($config{RSYNC_DEFAULT} eq 'TRUE') {
   %supported = (
-    '14.0' => 'rsync://slackbuilds.org/slackbuilds/14.0/',
-    '14.1' => 'rsync://slackbuilds.org/slackbuilds/14.1/',
-    '14.2' => 'rsync://slackbuilds.org/slackbuilds/14.2/',
     '15.0' => 'rsync://slackbuilds.org/slackbuilds/15.0/',
     '15.0+' => 'https://github.com/Ponce/slackbuilds.git',
     '15.1' => 'rsync://slackbuilds.org/slackbuilds/15.1/',
@@ -631,9 +632,6 @@ if ($config{RSYNC_DEFAULT} eq 'TRUE') {
 }
 
 my %branch = (
-  '14.0' => '14.0',
-  '14.1' => '14.1',
-  '14.2' => '14.2',
   '15.0' => '15.0',
   '15.1' => '15.1',
 );
@@ -672,6 +670,8 @@ sub get_slack_version {
     close $fh;
     $version = ($line =~ /\s+(\d+[^\s]+)$/)[0];
   }
+  usage_error("\n$version is no longer supported. Use sbotools-4.0.1.\n" .
+    "Alternatively, set SLACKWARE_VERSION to 15.0 or greater.") if $version =~ /^14\./;
   usage_error("\nThe running or configured Slackware version is unsupported: $version\n" .
     "Consider running \"sboconfig -r $supported{current}\" to use a repository for -current.")
     unless $supported{$version};
